; -----------------------------------------------------------------------------
; Hunchy II - v1.0 - Atari 2600 VCS - 4KB ROM
; Copyright (C) 2005 - Chris Walton (cd-w) <cwalton@gmail.com>
; Graphics - Juston Hairgrove (jussts)
; Music & Endgame Sequence - John Payson (supercat)
; -----------------------------------------------------------------------------

        PROCESSOR 6502
        INCLUDE vcs.h
        INCLUDE macro.h
	
        ; BIT Absolute Macro
        MAC BITABS
        .byte $2C
        ENDM

; Game Constants
NO_ILLEGAL_OPCODES = 1          ; No Cheating :-)
PALCOLS         = 0             ; PAL/NTSC Colours (0 = NTSC)
PALTIME         = 0             ; 50Hz/60Hz Timing (0 = 60Hz)
TUNELENGTH	= 95		; Music Length
		
; Screen Constants
KERNH		= 96		; Screen Height
MAXROW		= 12		; Platform Rows
	
; Player Constants
STARTLIVES	= 4		; Starting Lives
PHEIGHT		= 11		; Sprite Height
XMIN		= 17		; Minimum X Position
XMAX		= 136		; Maximum X Position
STARTX		= 30		; Starting X Coordinate
STARTY		= 40		; Starting Y Coordinate
JUMPHEIGHT	= 10		; Jump Height (Pixels)
		
; Knight Constants
KHEIGHT		= 13		; Sprite Height (Normal)
KHEIGHT2	= 17		; Sprite Height (with Spear)
	
; Wall Constants
WHEIGHT		= 8		; Tile Data Length

; Knight Delay
KDELAY		= 2		; Delay Factor

; Missile Constants
MDELAY		= %00011111	; Missile Firing Delay
MISSILEMAX      = 67            ; Maximum X Position
MISSILEMAX2 = MISSILEMAX + 128  ; Set Bit 7 
MISSILEMIN      = 4             ; Minimum X Position
MISSILEMIN2 = MISSILEMIN + 128  ; Set Bit 7	
			
        IF PALCOLS
; PAL Colours
BLACK           = $00
WHITE           = $0E
RED             = $62
GREEN           = $56
BLUE            = $D6
YELL            = $26
GREY            = $04
PURPLE          = $84
ORANGE          = $44
PINK            = $4C
LIGHTRED	= $64	
LIGHTBLUE       = $DE	
DARKBLUE        = $D4	
DARKGREEN       = $54
CYANGREEN	= $74
BROWN		= $24
MAGENTA		= $84
        ELSE    
; NTSC Colours
BLACK           = $00
WHITE           = $0C
RED             = $42
GREEN           = $C6
BLUE            = $84
YELL            = $18
GREY            = $04
PURPLE          = $50
ORANGE          = $24
PINK            = $3C
LIGHTRED	= $44	
LIGHTBLUE       = $7E		
DARKBLUE        = $92	
DARKGREEN       = $D4
CYANGREEN	= $B4
BROWN		= $F4
MAGENTA		= $54	
        ENDIF

        SEG.U VARS
        ORG $80

; General Game Variables
CYCLE		ds 1		; Game Cycle
TIMER		ds 1		; Bonus Timer (Bits 4-0)
BONUS		ds 1		; Bonus Value
RANDOM		ds 1		; Random Number Seed/Value
SWCH            ds 1            ; Console Switch Status	
DUR0		ds 1		; Sound Duration (Channel 0)
NOTE		ds 1		; Music Note Pointer
NTIME		ds 1		; Note Delay
NVOL		ds 1		; Music Volume
FDELAY		ds 1		; Fire Button Delay
	
; Screen Variables
SCREEN		ds #MAXROW*4	; Screen Data
ROW		ds 1		; Current Row (0->11)
END		ds 1		; Temp Screen Variable
SPTR0		ds 2		; PF1 Pointer (LHS)
SPTR1		ds 2		; PF2 Pointer (LHS)
SPTR2		ds 2		; PF2 Pointer (RHS)
SPTR3		ds 2		; PF1 Pointer (RHS)
PPTR		ds 2		; Player Sprite Pointer
KPTR		ds 2		; Knight Sprite Pointer	
MPTR		ds 2		; Missile Sprite Pointer
BPTR		ds 2		; Bell Sprite Pointer	
	
; Player Variables
LEVEL		ds 1		; Level (16->0), Bit 7 = Display Score
LIVES		ds 1		; Lives (7->0)
SCORE		ds 2		; High Score (4 Digit BCD)	
PLAYERX		ds 1		; Player X Coordinate 
PLAYERY		ds 1		; Player Y Coordinate
JSTEP		ds 1		; Jump Step
COLLIDE		ds 1		; Collision Vector
PSWITCH		ds 1		; Player SwitchDraw Start
PEND		ds 1		; Player SwitchDraw End
	
; Knight Variables
KNIGHTX		ds 1		; Knight X Coordinate
KNIGHTY		ds 1		; Knight Y Coordinate	
KSWITCH		ds 1		; Knight SwitchDraw Start
KEND		ds 1		; Knight SwitchDraw End
KMOVE		ds 1		; Knight Movement Vector	
KRESIZE		ds 1		; Resize Line
KNUSIZ0		ds 1		; Resize Value 1
KNUSIZ1		ds 1		; Resize Value 2
KDEL		ds 1		; Delay Counter
				
; Missile Variables
MISSILE		ds #MAXROW	; Missile Positions (0 = No Missile)
MSHAPE		ds 1		; Missile Shape (Pointer LSB)
MCOUNT		ds 1		; Current Number of Missiles
MDIR		ds 1		; Max (7-5), Delay (4-3) & Direction (3-0)
MDEL		ds 1		; Missile Delay	Value
MTIME		ds 1		; Missile Delay Counter

; Bell Variables
BELL		ds #MAXROW	; Bell Positions (0 = No Bell)
BELLF		ds 1		; Bell Animation Frame
BELLS		ds 1		; Remaining Bells

        ; Display Remaining RAM
        echo "----",($100 - *) , "bytes left (RAM)" 

; Begin code segment (4K ROM)
        SEG CODE
        ORG $F000

; -----------------------------------------------------------------------------
; Game Initialisation
; -----------------------------------------------------------------------------
        
Start
        ; Wipe Registers & Memory
        CLEAN_START
	
	; Reset Game Variables
        jsr ResetGame
		
MainLoop
        ; Do Vertical Sync (VSync Routine by Manuel Polik)
        lda #2                  ; VSYNC enable
        sta WSYNC 
        sta VSYNC 
        sta WSYNC  
        sta WSYNC 
        lsr                     ; VSYNC disable
        sta WSYNC  
        sta VSYNC
	
        ; Set Vertical Blank Timer (PAL/NTSC)
        IF PALTIME
        ldy #53                 ; (45*76)/64 = 53
        ELSE
        ldy #43                 ; (37*76)/64 = 43
        ENDIF
        sty TIM64T

; -----------------------------------------------------------------------------
; Main Game Logic (Vertical Blank)
; -----------------------------------------------------------------------------
	
GameLogic
 	; Check Console Switches
  	ldy #0
  	lda SWCHB
  	cmp SWCH
  	beq EndSwitches
  	sta SWCH
	; Don't Allow Select on Level >=14
	lda LEVEL
	cmp #14
	bcs CheckReset
  	; Check Select Switch (Practise Mode)
  	lda #%00000010
  	bit SWCH
 	; Only Permit Practise Mode If Left Difficulty Switch Set
 	bvc CheckReset
  	bne CheckReset
  	; Skip Level
  	sty BELLS
  	sty TIMER
CheckReset
  	; Check Reset Switch
  	lda #%00000001
   	bit SWCH
   	bne EndSwitches
   	; Reset Game
   	sty LIVES
   	jsr ResetGame
EndSwitches

	; Update Game Cycle & Timer
        dec CYCLE
	bne EndTime
	; Decrease Level Timer On Cycle Wrap (If > 0)
	lda TIMER
	beq EndTime
	dec TIMER
EndTime
	
	; Increment Score (BONUS > 0)
	lda BONUS
	beq EndBonus
	dec BONUS
	sed
	clc
 	lda SCORE+1
 	adc #5
	sta SCORE+1
	lda SCORE
	adc #0
	sta SCORE
 	cld	
EndBonus

	; Display Score Screen (LEVEL Bit 7 Set)
 	lda LEVEL
  	bpl Continue
 	jmp DisplayScore
Continue
	
	; Check Remaining Bells (0 = Next Level)
	lda BELLS
	bne EndBellCheck
NewLevel
	; Store Timer Value As Bonus
	lda TIMER
	sta BONUS	
	; Increment Level	
	inc LEVEL
	; Display Score Screen
	bne ShowTitle
EndBellCheck
		
	; Check If Player Is Dead
	lda COLLIDE
	lsr
	bcc EndDead
	; Play Falling Sound
	lda #KERNH+5
	sbc PLAYERY
	lsr
	lsr
        sta AUDF0 
	lda #8
        sta AUDV0
        lsr
        sta AUDC0	
	inc DUR0
	; Decrease Position & Check If At Bottom of Screen
	dec PLAYERY
	bne NoMove
	; Decrease Lives
	dec LIVES
ShowTitle
	lda LEVEL
	; Wrap At Level 16
	and #%00001111
	; Set Bit 7 To Display Title Screen
	ora #%10000000	
	sta LEVEL
	; Delay Fire Button
	lda #90
	sta FDELAY
NoMove
	; Don't Allow Player To Move
	jmp EndPlayerMove
EndDead
		
NoFinal
	; Move Every Second Cycle (Customise Later?)
	lda CYCLE
	lsr
	bcs NoMove
	; Automatic Movements On Level 15 
	lda LEVEL
	and #%00001111
	cmp #15
	bne RightJoy
	; Can't Move Right Through Blocks
	bit COLLIDE
	bpl NoMove	
	; Movements Synchronised To Music
	lda NOTE
        cmp #24			; Move Left After 24
        bcc LeftAuto
        cmp #48			; Dont Move Between 48 and 24
        bcc NoMove
RightAuto			; Move Right Before 48
	; Move Right
        inc PLAYERX
        bne NoMove
LeftAuto:
        lda #8
        sta REFP0
        cmp PLAYERX
        bcs NoMove
        dec PLAYERX
        bne NoMove	
RightJoy
        ; Move Right
	bit COLLIDE
	bpl LeftJoy
	bit SWCHA
	bmi LeftJoy
	; Increment Position
        inc PLAYERX
        ; Sprite Not Reflected
        lda #0
	beq HorizMove
LeftJoy
	; Move Left	
 	bit COLLIDE
 	bvc DownJoy	
	bit SWCHA
	bvs DownJoy
        ; Decrement Position
        dec PLAYERX
        ; Reflect Sprite
        lda #8
HorizMove
	; Set Sprite Reflection
        sta REFP0
	; Clear Ladder Bit
	lda #%11111101
	and COLLIDE
	sta COLLIDE	
	; Play Sound
	lda #6
        sta AUDF0
        sta AUDV0
        asl
        sta AUDC0
        inc DUR0
DownJoy
	; Move Down Ladder	
	lda #%00100000
	and COLLIDE
	beq UpJoy
	bit SWCHA
	bne UpJoy
	dec PLAYERY
	bne VertMove
UpJoy
	; Move Up Ladder	
	lda #%00010000
	and COLLIDE
	beq EndJoy
	bit SWCHA
	bne EndJoy
	inc PLAYERY
VertMove	
	; Set Ladder Bit
	lda #%00000010
	ora COLLIDE
	sta COLLIDE	
	; Play Sound
        lda #4
        sta AUDF0
	lda #6
        sta AUDV0	
        asl
        sta AUDC0       
        inc DUR0		
EndJoy

JumpJoy
	; Check For Top Collisions
 	lda #%00001000
 	and COLLIDE
 	beq StopJump
        ; Check If Already Jumping
        lda JSTEP
        beq NewJump
	; Play Jumping Sound
	sec
	lda #JUMPHEIGHT+4
	sbc JSTEP
        sta AUDF0
	lda #6
        sta AUDV0
        asl
        sta AUDC0	
        inc DUR0	
        ; Increment Jump Step
        inc JSTEP
        ; Check Jump End
	lda JSTEP	
        cmp #JUMPHEIGHT
	beq EndJump		; Pause At Jump Apex
 	bcs StopJump		; Finish Jump
	inc PLAYERY
	bcc EndJump
NewJump
	; Check If Jump Allowed
	lda #%00000100
	and COLLIDE
	bne EndJump
	; Read Fire Button
        lda INPT4
        bmi EndJump
StartJump
	; Clear Ladder Bit
	lda #%11111101
	and COLLIDE
	sta COLLIDE
	; Start Jumping
	lda #1
	BITABS
StopJump
	; Stop Jumping
	lda #0
	sta JSTEP
EndJump

FallDown
	; Don't Fall If Jumping
	lda JSTEP
	bne EndFall
	; Check Collision Vector
	lda #%00000100
	and COLLIDE
	beq EndFall
	; Decrease Position
	dec PLAYERY
	; Play Falling Sound
	lda #KERNH+5
	sbc PLAYERY
	lsr
	lsr
        sta AUDF0 
	lda #6
        sta AUDV0
        asl
        sta AUDC0	
        inc DUR0	
	; Clear Ladder Bit
 	lda COLLIDE
  	and #%11111101
  	sta COLLIDE
EndFall
EndPlayerMove

	; Set Default Missile Delay
	lda #MDELAY
	sta MDEL
	
	; Silence Sound Effects
        lda DUR0
        bne Dec0
        sta AUDV0
        BITABS                  ; Skip Next Instruction
Dec0
        dec DUR0
	
	; Set Low AUDCx Value (Default)
	ldy #1
	IF PALTIME
	ldx #9
	ELSE
	ldx #11
	ENDIF
	; Play Music On Level 0 & 15
	lda LEVEL
	and #%00001111
	beq PlayMusic
	cmp #15
	bne MusicDone
	; High AUDCx Value on Level 15	
	ldy #4
	IF PALTIME
	ldx #6
	ELSE
	ldx #7
	ENDIF
PlayMusic
	; Set AUDCx
	sty AUDC1
	; Speed Up Missiles
	lsr MDEL
	; Play Music & End Sequence
	jsr PlaySequence
MusicDone	
			
	; Move Knight If Following Player (KMOVE Bit 0 Set)
	lda KMOVE
	and #%00000001
	beq EndKnight
KFollow	
	; Check If Knight Is Falling
	lda #%00001000
	and KMOVE
	beq EndKFall	
	; Fall Every Second Cycle
	lda CYCLE
	lsr
	bcs EndKnight
	dec KNIGHTY
	bcc EndKnight
EndKFall
	; Delay Knight Movement by KDEL Frames
	dec KDEL
	bpl EndKnight
	lda #KDELAY
	sta KDEL
	; Move Knight Up Ladder
KnightUp
	lda #%00010000
	and KMOVE
	beq KnightDown
	ldy KNIGHTY
	dey
	dey
	cpy PLAYERY
	bcs KnightDown
	inc KNIGHTY
	bne EndKnight
	; Move Knight Down Ladder
KnightDown
	lda #%00100000
	and KMOVE
	beq KnightLeft
	ldy PLAYERY
	iny
	iny
	cpy KNIGHTY
	bcs KnightLeft
	dec KNIGHTY
	bne EndKnight
	; Move Knight Left
KnightLeft
	lda #%01000000
	and KMOVE
	beq KnightRight		
	lda PLAYERX
	cmp KNIGHTX
	bcs KnightRight
	dec KNIGHTX
	lda #0
        sta REFP1	
	beq EndKnight
	; Move Knight Right
KnightRight
	lda #%10000000
	and KMOVE
	beq EndKnight
	lda KNIGHTX
	cmp PLAYERX
	bcs EndKnight
	inc KNIGHTX
	lda #8
        sta REFP1	
EndKnight		

	; Missile Firing Patterns:
	; 0 = No Missiles
	; 1 = Wall of Missiles (LHS)
	; 2 = Wall of Missiles (RHS)
	; 3 = Staggered Missiles (LHS)
	; 4 = Staggered Missiles (RHS)
	; 5 = Random Missiles (LHS)
	; 6 = Random Missiles (RHS)
	; 7 = Random Missiles (Bi-directional)

	; Delay Missile Fire (Otherwise All Missiles Synchronised)
        lda MDEL
        bit CYCLE
        bne EndNewMissiles	
	; Skip Firing If No Missiles
	lda MDIR
	and #%00000111
	beq EndNewMissiles
	; Check Firing Pattern
	cmp #5
	bcs RandomMissiles
	cmp #3
	bcs StaggeredMissiles
	; Pattern 1 - Wall of Missiles
WallOfMissiles
RWall
	cmp #1
	beq LWall
	lda #MISSILEMAX2
	BITABS
LWall
        lda #MISSILEMIN
StoreWall 
        ldy #MAXROW-1
WallLoop
        ; Check Missile Not Already Fired
	ldx MISSILE,Y
	bne NextW
        sta MISSILE,Y	
NextW	
	dey
	bpl WallLoop
	bmi EndNewMissiles
	; Pattern 2 - Staggered Missiles
StaggeredMissiles
RStagger
	cmp #3
	beq LStagger
        lda #MISSILEMAX2
	BITABS
LStagger
        lda #MISSILEMIN
        ldy #MAXROW-1
StaggerLoop
        ; Check Missile Not Already Fired
	ldx MISSILE,Y
	beq EndStaggerLoop		
	dey
	bpl StaggerLoop
	bmi EndNewMissiles
EndStaggerLoop
	sta MISSILE,Y
	beq EndNewMissiles
	; Pattern 3 - Random Missiles
RandomMissiles
	; Check If Maximum Count Reached
	lda MDIR
	lsr
	lsr
	lsr
	lsr
	lsr
	cmp MCOUNT
        beq EndNewMissiles
	; Start New Missiles
NewMissile
	; Get Random Number 1 < RAND < 11
        jsr Random128
	and #%00000111		; Get Number in Range 0->7
	sta END	
	jsr Random128
	and #%00000011		; Add Number in Range 0->3
	sec
	adc END
        ; Check if Missile Position Already Taken
        tay     
        lda MISSILE,Y
        bne EndNewMissiles
        ; Set Missile Direction
	lda MDIR
	and #%00000111
RMissile
	; Missile From RHS
	cmp #6
	bne LMissile
	lda #MISSILEMAX2
	bne StoreNewMissile
LMissile
	; Missile From LHS
	cmp #5
	bne BMissile
        lda #MISSILEMIN
	bne StoreNewMissile
BMissile
	; Set Direction According to Bit 7 of RANDOM 
        lda #MISSILEMIN	
        bit RANDOM
        bpl StoreNewMissile
        lda #MISSILEMAX2
StoreNewMissile
        sta MISSILE,Y
        inc MCOUNT
EndNewMissiles
	
	; Delay Missile Movement
	dec MTIME
	bpl EndMissileLoop
	; Reset Delay Counter
	lda MDIR
	and #%00011000
	lsr
	lsr
	lsr
	sta MTIME	
        ; Move All Missiles 
        ldy #MAXROW-1
MissileLoop
        ; Check Missile Fired
        ldx MISSILE,Y
        beq NextMissile
        ; Check Direction
        bmi DecMissile
        ; Increment Position
IncMissile
        inx
        cpx #MISSILEMAX
        bne StoreMissile
        beq ResetMissile
DecMissile
        ; Decrement Position
        dex
        cpx #MISSILEMIN2
        bcs StoreMissile
ResetMissile
        ; Decrement Counter
        dec MCOUNT
	; Reset Position
        ldx #0
StoreMissile
        stx MISSILE,Y
NextMissile
        dey
        bpl MissileLoop
EndMissileLoop
		
	; Set Missile Shape
	lda MDIR
	and #%00000111
	tax
	lda MissileShapes,X
	sta MSHAPE

	; Display Bell or Lever
	lda LEVEL
        cmp #15
        bne NormBell
	; Flip Lever At Note 77
	ldx NOTE
	cpx #77
	bcc FlipLever
        lda #<Lever1
        bne StoreBell
FlipLever
	lda #<Lever2
	bne StoreBell
	; Update Bell Animation Frame	
NormBell
        lda CYCLE
        lsr
        lsr
        lsr
        and #%00000111
        tax
        lda BellFrames,X
StoreBell
        sta BELLF                  	
		
	; Reset Sprite Pointers (MSB)
	lda #>StartSprites1
	sta PPTR+1
	sta MPTR+1
	sta BPTR+1		
	lda #>StartSprites2
	sta KPTR+1

	; Reset Screen Pointers
	ldy #6
	ldx #<Tile0-WHEIGHT-4
	lda #>Tile0
SptrLoop
	sta SPTR0+1,Y
	stx SPTR0,Y
	dey
	dey
	bpl SptrLoop
	
	; Set Player Sprite Colour
	lda #YELL
	sta COLUP0
	
        ; Calculate SwitchDraw Start For Player
        sec
	lda PLAYERY
	tax
	sbc #PHEIGHT
	sta PEND
	; Fix Top Line Problem (Copy PEND to PSWITCH if at top of screen)
	cpx #KERNH
	bcc NoCopy
	ora #%10000000
	tax
NoCopy
	stx PSWITCH
	        	   
        ; Calculate Player Sprite Pointer (Animated)
	lda COLLIDE
	and #%00000011		; Check If On Ladder Or Dead
	beq HorizFrame
	; Player Moving Vertically
VertFrame
	lda PLAYERY
	and #%00000100
	lsr
	lsr
	tax
	lda VertFrames,X
	bne EndFrame
	; Player Moving Horizontally
HorizFrame
        lda PLAYERX
	and #%00001100
	lsr
	lsr
	tax
	lda HorizFrames,X
EndFrame
	; Fix Pointer For SwitchDraw
	sec
        sbc PEND
        sta PPTR

	; Calculate SwitchDraw Start For Knight
        lda KNIGHTY
        sta KSWITCH
	sec
        sbc #KHEIGHT
        sta KEND	
		
	; Show Esmeralda Or Knight (Bit 2 of KMOVE)
	lda KMOVE
	and #%00000100
	beq DisplayKnight
DisplayEsmeralda
	; Set Esmeralda Sprite Colour
	lda #RED
	sta COLUP1
	; Fix Sprite Start Pointer
	lda #>StartSprites1
	sta KPTR+1	
	; Calculate Esmeralda Sprite Pointer (Animated)
	lda CYCLE
	and #%00011000
	lsr
	lsr
	lsr
	tax
	lda DamselFrames,X
	bne EndDisplayKnight
DisplayKnight
	; Set Knight Sprite Colour
	lda #BLUE
	sta COLUP1	
	; Check if Knight Is Thrusting (KMOVE Bit 1 Set)
	lda KMOVE
	and #%00000010
	beq MovingKnight
ThrustingKnight
	; Calculate SwitchDraw Start For Knight (Sprite Is Taller)
        lda KNIGHTY
	sec
        sbc #KHEIGHT2
        sta KEND
	; Calculate Animation Frame
	lda CYCLE
	and #%00110000
	lsr
	lsr
	lsr
	lsr
	tax
	lda KnightSpearFrames,X
	bne EndDisplayKnight
MovingKnight
	; Vertical/Horizontal Frame
	lda KMOVE
	and #%00110000
	beq KHorizFrame
KVertFrame
	lda KNIGHTY
	and #%00000100
	lsr
	lsr
	tax
	lda KnightVertFrames,X
	bne EndDisplayKnight
KHorizFrame
	lda KNIGHTX
	and #%00000110
	lsr
	tax
	lda KnightHorizFrames,X
EndDisplayKnight
	sec
	sbc KEND
	sta KPTR

	; Fix-up SwitchDraw End Pointers
	lda PEND
        ora #%10000000
        sta PEND	
	lda KEND
        ora #%10000000
        sta KEND	
							
        ; Position Player Sprite (Player 0)
        ldx #0
        lda PLAYERX
        jsr XPosition
        ; Position Soldier Sprite (Player 1)
        lda KNIGHTX
        jsr XPosition

        ; Commit Initial Sprite Movements
        sta WSYNC
	sta HMOVE	
	
	; Set Player 1 Sprite Size (Knight/Damsel)
	lda KNUSIZ0
	sta NUSIZ1
	
        ; Reset Collisions
        sta CXCLR

	; Set Starting Row
	lda #MAXROW-1
	sta ROW
		
	; Set Display Height
	ldx #0
	ldy #KERNH	
	
        ; Wait for Vertical Blank End
WaitVblank
        lda INTIM
        bne WaitVblank
        sta VBLANK	
	sta WSYNC

	; Reset Sprite Movements
        sta HMCLR
	
	; Reflect PF & Draw Side Walls (To Hide Sprites) 
	lda #%11000101
	sta CTRLPF
	sta PF0
				
	; Draw Screen
	jmp Kernel1A

	; Random Number Generation Routine
        ; On Exit: A = Random Number
Random128
        lda RANDOM
        asl                     
        bcc NoEor  
        eor #$CF
NoEor
        sta RANDOM   
        rts
	
; -----------------------------------------------------------------------------
; Display Kernels
; -----------------------------------------------------------------------------
		
	ALIGN	256
		
	; Player SwitchDraw Routines (By Thomas Jentzsch)
PSwitch1
        lda PEND                ; [6] + 3      
        sta.w PSWITCH           ; [9] + 4
        bne PContinue1          ; [13] + 3
PWait1
        SLEEP 5                 ; [8] + 5
        jmp PContinue1          ; [13] + 3
	; Knight SwitchDraw Routines (By Thomas Jentzsch)
KSwitch1
        lda KEND                ; [63] + 3      
        sta KSWITCH             ; [66] + 3
        bne KContinue1          ; [69] + 3
KWait1
	SLEEP 4			; [65] + 4
        jmp KContinue1          ; [69] + 3
	
; LINE 0 - REPOSITION BELL
Kernel1A
	; Draw Bar At Top Of Screen
	lda #$FF
	sta PF1
	sta PF2
	sta WSYNC
Kernel1
	; Disable Bell
	stx ENAM0		; [0] + 3
	
	; Fetch Bell Position
	ldx ROW			; [3] + 2
	lda BELL,X		; [5] + 4
        ; Divide Position by 15
	sec			; [9] + 2
BDiv15
        sbc #15                 ; [11] + 2
        bcs BDiv15              ; [13/18/23/28/33/38/43/48/53] + 2/3
        ; Store Position
	tax			; [55] + 2
        lda FineTuneEnd,X       ; [57] + 5*
	sta.w RESM0		; [62] + 3		
        sta HMM0                ; [65] + 3
	
	; Decrease Line Counter
	dey			; [68] + 2		
			
; LINE 1 - DISPLAY PLAYER
	sta WSYNC		; [0]
	
	; Draw Player
	cpy PSWITCH             ; [0] + 3
        beq PSwitch1            ; [3] + 2/3
        bpl PWait1              ; [5] + 2/3
        lda (PPTR),Y            ; [7] + 5
        sta.w GRP0		; [12] + 4
PContinue1

	; Draw Screen
	lda (SPTR0),Y		; [16] + 5
	sta PF1			; [21] + 3	< 28
	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3	< 38
	lda (SPTR3),Y		; [32] + 5
	sta PF1			; [37] + 3      > 38 < 60
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT

	; Calculate Wall Position
	sec			; [48] + 2
        tya			; [50] + 2
        sbc #WHEIGHT+1		; [52] + 2
        sta END			; [54] + 3
	
	; Draw Knight
	cpy KSWITCH             ; [57] + 3
        beq KSwitch1            ; [60] + 2/3
        bpl KWait1              ; [62] + 2/3
        lda (KPTR),Y            ; [64] + 5
        sta GRP1                ; [69] + 3	> 70
KContinue1

	; Fetch Display Row
	lda ROW			; [72] + 3
	
	; Commit Bell Movement
	sta HMOVE		; [75] + 3
	
	; Calculate Screen Offset (ROW * 4)
	asl			; [2] + 2	
	asl			; [4] + 2
	tax			; [6] + 2
	
; LINE 2 - UPDATE SCREEN POINTERS
	
	; Draw Screen
	lda (SPTR0),Y		; [8] + 5
	sta PF1			; [13] + 3	< 28
	lda (SPTR1),Y		; [16] + 5	
        sta PF2                 ; [21] + 3	< 38

	; Calculate SPTR0 Pointer
        sec			; [24] + 2
        lda SCREEN,X		; [26] + 4
        sbc END			; [30] + 3
        sta SPTR0		; [33] + 3

	nop			; [36] + 2
	sec			; [38] + 2
	
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT
	lda (SPTR3),Y		; [48] + 5
	sta PF1			; [53] + 3      > 38 < 60

	; Calculate SPTR1 Pointer
        lda SCREEN+1,X		; [56] + 4
        sbc END			; [60] + 3
        sta SPTR1		; [63] + 3

	; Calculate SPTR2 Pointer
        sec			; [66] + 2
        lda SCREEN+2,X		; [68] + 4
        sbc END			; [72] + 3
        sta SPTR2		; [75] + 3
		
	; Decrease Line Counter
	dey			; [2] + 2
	
; LINE 3 - DISPLAY PLAYER & CALCULATE SCREEN	
	; Draw Player
	cpy PSWITCH             ; [4] + 3
        beq PSwitch2            ; [7] + 2/3
        bpl PWait2              ; [9] + 2/3
        lda (PPTR),Y            ; [11] + 5
        sta GRP0		; [16] + 3
PContinue2

	; Clear PF
	lda #0			; [19] + 2
	sta PF1			; [21] + 3	< 28
	sta PF2			; [24] + 3	< 38
			
	; Calculate SPTR3 Pointer
	sec			; [27] + 2
        lda SCREEN+3,X		; [29] + 4
        sbc END			; [33] + 3
        sta SPTR3		; [36] + 3

	; Calculate Bell Pointer
        lda BELLF		; [39] + 3
        sbc END			; [42] + 3
        sta BPTR		; [45] + 3

	; Calculate Missile Pointer
	ldx ROW			; [48] + 3
	lda MSHAPE		; [51] + 3
        sbc END			; [54] + 3
        sta MPTR		; [57] + 3
	
	; Draw Knight
	cpy KSWITCH             ; [60] + 3
        beq KSwitch2            ; [63] + 2/3
        bpl KWait2              ; [65] + 2/3
        lda (KPTR),Y            ; [67] + 5
        sta GRP1                ; [72] + 3	> 70
KContinue2
	
	; Clear Sprite Movements
	sta HMCLR		; [75] + 3

	; Decrease Line Counter
	dey			; [2] + 2
		
; LINE 4 - REPOSITION MISSILE
		
	; Fetch Bell Position
	lda MISSILE,X		; [4] + 4
	asl			; [8] + 2
        ; Divide Position by 15
MDiv15
        sbc #15                 ; [10] + 2
        bcs MDiv15              ; [12/17/22/27/32/37/42/47/52] + 2/3
        ; Get Fine Position
        tax                     ; [54] + 2
        lda FineTuneEnd,X       ; [56] + 5*
	; Set Main Kernel Counter
	ldx #6			; [61] + 2	
	; Store Coarse Position
        sta RESM1		; [63] + 3	
        sta HMM1                ; [66] + 3
	jmp Kernel2		; [69] + 3
	
	; Player SwitchDraw Routines
PSwitch2
        lda PEND                ; [10] + 3      
        sta PSWITCH		; [13] + 3
        bne PContinue2          ; [16] + 3
PWait2
        SLEEP 4			; [12] + 4
        jmp PContinue2          ; [16] + 3

	; Knight SwitchDraw Routines
KSwitch2
        lda KEND                ; [63] + 3      
        sta KSWITCH             ; [66] + 3
        bne KContinue2          ; [69] + 3
KWait2
	SLEEP 4			; [65] + 4
        jmp KContinue2          ; [69] + 3

EndKernel1
	if (>Kernel1 != >EndKernel1)
          echo "WARNING: Kernel 1 crosses a page boundary!"
        endif	

        ; Sprite Horizontal Positioning Routine (Battlezone Variant)
        ; On Entry: A = Coordinate and X = Object ID
XPosition
        sta WSYNC               ; [0]
        sec                     ; [0] + 2
        ; Divide Position by 15
Divide15
        sbc #15                 ; [2] + 2
        bcs Divide15            ; [4] + 2/3
        nop                     ; [6] + 2
        tay                     ; [8] + 2
        ; Store Coarse and Fine Position
        lda FineTuneEnd,Y       ; [10] + 5*
        sta HMP0,X              ; [14] + 4
        sta RESP0,X             ; [18] + 4
        ; Position At Cycles: [22/27/32/37/42/47/52/57/62/67/72/77]
        inx
        rts
	
	ALIGN	256
	
	; Player SwitchDraw
PSwitch3
        lda PEND                ; [6] + 3      
        sta.w PSWITCH		; [9] + 4
        bne PContinue3          ; [13] + 3
PWait3
        SLEEP 5                 ; [8] + 5
        jmp PContinue3          ; [13] + 3
	
	; Knight SwitchDraw
KSwitch3
        lda KEND                ; [58] + 3      
        sta.w KSWITCH           ; [61] + 4
        bne KContinue3          ; [65] + 3
KWait3
        SLEEP 5			; [60] + 5
	jmp KContinue3          ; [65] + 3	

KSwitch4
        lda KEND                ; [54] + 3      
        sta KSWITCH             ; [57] + 3
        bne KContinue4          ; [60] + 3
KWait4
        SLEEP 4			; [56] + 4
	jmp KContinue4          ; [60] + 3	

	; New Line Routine
NewLine
	; Decrement Row
	dec ROW			; [53] + 5
	; Draw Knight
	cpy KSWITCH             ; [58] + 3
        beq KSwitch4            ; [61] + 2/3
        bpl KWait4              ; [63] + 2/3
        lda (KPTR),Y            ; [65] + 5
        sta GRP1		; [70] + 3	> 70
KContinue4
	; Start Kernel Again
	jmp Kernel1		; [73] + 3

	; Missile Draw
DrawMissile
 	lda (MPTR),Y		; [57] + 5
 	sta HMM1		; [62] + 3
	jmp EndResize		; [65] + 3
		
Kernel2
	sta WSYNC		; [0]
	
; LINE 5-15 DISPLAY SPRITES & SCREEN (MAIN KERNEL LOOP)
KernelLoop
	; Draw Player
	cpy PSWITCH             ; [0] + 3
        beq PSwitch3            ; [3] + 2/3
        bpl PWait3              ; [5] + 2/3
        lda (PPTR),Y            ; [7] + 5
        sta.w GRP0		; [12] + 4
PContinue3
		
	; Draw Screen
	lda (SPTR0),Y		; [16] + 5
	sta PF1			; [21] + 3	> 70 < 28
	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3      > 60 < 38
	lda (SPTR3),Y		; [32] + 5
	sta PF1			; [37] + 3      > 38 < 60
	lda (SPTR2),Y		; [40] + 5
	sta PF2			; [45] + 3      = 48 EXACT

	; Check Row
	dex			; [48] + 2
	beq NewLine		; [50] + 2/3

	; Draw Knight
	cpy KSWITCH             ; [52] + 3
        beq KSwitch3            ; [55] + 2/3
        bpl KWait3              ; [57] + 2/3
        lda (KPTR),Y            ; [59] + 5
        sta.w GRP1		; [64] + 4	SHOULD BE > 70 !!!
KContinue3
	
	; Move Bell
 	lda (BPTR),Y		; [68] + 5
	sta HMM0		; [73] + 3
	
	; Commit Movements
	sta HMOVE		; [0] + 3

	; Draw Bell (M0)
	sta ENAM0		; [3] + 3	
	asl			; [6] + 2
 	asl			; [8] + 2
	sta NUSIZ0		; [10] + 3
	SLEEP 3			; [13] + 3

	; Draw Screen
 	lda (SPTR0),Y		; [16] + 5
 	sta PF1			; [21] + 3	< 28
 	lda (SPTR1),Y		; [24] + 5	
        sta PF2                 ; [29] + 3      < 38
 	lda (SPTR3),Y		; [32] + 5
 	sta PF1			; [37] + 3      > 38 < 60
 	lda (SPTR2),Y		; [40] + 5
 	sta PF2			; [45] + 3      = 48 EXACT
	dey			; [48] + 2
	
	; Draw Missiles/Resize Knight Sprite
	cpy KRESIZE		; [50] + 3
	bpl DrawMissile		; [53] + 2/3
	lda KNUSIZ1		; [55] + 3
	sta NUSIZ1		; [58] + 3
	lda #0			; [61] + 2
	SLEEP 4			; [63] + 4
EndResize
	sta.w ENAM1		; [67] + 4
	
	; Check End Condition
	cpy #1			; [71] + 2
	bne KernelLoop		; [73] + 2/3
	dey			; [75] + 2

	; Clear PF0, Sprite & Bell Graphics
	sty PF0			; [1] + 3
	sty GRP0		; [4] + 3
	sty ENAM0		; [7] + 3	
		
	; Set PF Score Bit To Display Split Colours
	ldy #%00000011		; [10] + 2
	sty CTRLPF		; [12] + 3
	nop			; [15] + 2
LivesLoop
	; Display Lives
	ldx LIVES		; [17] + 3
	lda Lives1,X		; [20] + 4
	sta PF1			; [24] + 3	< 28
	lda Lives2,X		; [27] + 4
	sta PF2			; [31] + 3	< 38
	; Display Bonus Timer
	lda TIMER		; [34] + 3
	lsr			; [37] + 2
	tax			; [39] + 2
	lda TimeBar2,X		; [41] + 4
	sta PF2			; [45] + 3	= 48 EXACT
	lda TimeBar1,X		; [48] + 4
	sta PF1			; [52] + 3	> 38 < 60	
	sta WSYNC		; [0]
	SLEEP 12		; [0] + 12
	dey			; [12] + 2
	bne LivesLoop		; [14] + 2/3
EndLives
	; Clear PF 
	sty PF1			; [14] + 2	< 28
	sty PF2			; [16] + 2	< 38
		 
	; Skip PAL Lines
	IF PALTIME
	ldy #35
SpaceLoop
	sta WSYNC
	dey
	bne SpaceLoop
	ENDIF
	                   
        ; Start Vertical Blank
        lda #2
        sta WSYNC
        sta VBLANK
	
        ; Set Timer for Over-scan
        IF PALTIME
        ldy #42                 ; (36*76)/64 = 42
        ELSE
        ldy #35                 ; (30*76)/64 = 35
        ENDIF
        sty TIM64T

CheckDead
	; Player Missile Collision
	lda CXM1P
	bmi Die
	; Screen Bottom Collision
	lda PLAYERY
	cmp #PHEIGHT
	bcc Die
	; Skip Knight Collision If Displaying Esmeralda (KMOVE Bit 2 Set)
	lda KMOVE
	and #%00000100
	bne EndCheckDead
	; Player Knight Collision	
	lda CXPPMM
	bpl EndCheckDead	
Die
	; Set Dead Bit
	lda COLLIDE
	ora #%00000001
	sta COLLIDE
EndCheckDead
			
	; Jump To Collision Detection Routines
        jmp CollisionDetect
		
EndKernel2
        if (>Kernel2 != >EndKernel2)
          echo "WARNING: Kernel 2 crosses a page boundary!"
        endif	
	
	ALIGN	256
			
	; Display 48-Pixel (6 Char) Score
ScoreKernel
	sta WSYNC		; [0]
	ldy #9			; [0] + 2
	sty ROW			; [2] + 3
	; Set Delay and Three Copies
	ldy #3			; [5] + 2
	sty VDELP0		; [7] + 3
	sty VDELP1		; [10] + 3
	sty NUSIZ0		; [13] + 3
	sty NUSIZ1		; [16] + 3
	; Set Colour
	lda #WHITE		; [19] + 2
	sta COLUP0		; [21] + 3
	sta COLUP1		; [24] + 3
	; Clear Any Sprite Reflections
	lda #%00010000          ; [27] + 2
	sta REFP0		; [29] + 3
	sta REFP1		; [32] + 3
	; Position Sprites
	sta HMP1		; [35] + 3
	sta RESP0		; [38] + 3
	sta RESP1		; [41] + 3
        ; Fine Tune Position of P1
        sta HMP1                ; [44] + 3
        iny			; [47] + 2
        lda (SPTR0),Y           ; [49] + 5
        sta GRP0                ; [54] + 3
        lda (SPTR1),Y           ; [57] + 5
        sty WSYNC               ; [0] 
        sty HMOVE               ; [0] + 3
        bne StartScore          ; [3] + 3
ScoreLoop
	; Fetch Current Line
	lda ROW			; [62] + 3
 	lsr			; [65] + 2	2LK
 	tay			; [67] + 2
	; Display First 3 Digits
	lda (SPTR0),Y		; [69] + 5
	sta GRP0		; [74] + 3	> 54
	lda (SPTR1),Y		; [1] + 5
StartScore
	sta GRP1		; [6] + 3	< 42
	lda (SPTR2),Y		; [9] + 5
	sta GRP0		; [14] + 3	< 44
	; Pre-fetch Remaining 3 Digits
	lda (SPTR3),Y		; [17] + 5
	tax			; [22] + 2
	lda (PPTR),Y		; [24] + 5
	sta END			; [29] + 3
	lda (KPTR),Y		; [32] + 5
	tay			; [37] + 2
	lda END			; [39] + 3
	; Display Remaining 3 Digits
	stx GRP1		; [42] + 3	> 44 < 47
	sta GRP0		; [45] + 3	> 46 < 50
	sty GRP1		; [48] + 3	> 49 < 52
	sta GRP0		; [51] + 3	> 52 < 55
	; Update Counter
	dec ROW			; [54] + 5
	bpl ScoreLoop		; [59] + 2/3
	; Clear Everything
 	lda #0			; [61] + 2
	sta GRP0		; [63] + 3
	sta GRP1		; [66] + 3
	sta GRP0		; [69] + 3
 	sta VDELP0		; [72] + 3
	sta VDELP1		; [75] + 3
	rts
			
; -----------------------------------------------------------------------------
; Collision Detection (Over-scan)
; -----------------------------------------------------------------------------
	
CollisionDetect
	; Construct Player Collision Vector COLLIDE
	; Bit 7 - Can Move Right
	; Bit 6 - Can Move Left
	; Bit 5 - Can Move Down Ladder
	; Bit 4 - Can Move Up Ladder
	; Bit 3 - Can Jump
	; Bit 2 - Fall
	; Bit 1 - On Ladder
	; Bit 0 - Dead
	
	; Reset Collision Vector
	lda COLLIDE
	and #%00000011		; Reset Bits 7-2
	ora #%11001100		; Set Defaults
	sta COLLIDE
	
	; Check If Already Dead
	and #%00000001
	beq CheckRight
	jmp EndPlayerColl
	; Check Player RHS
CheckRight
        ; Check Screen Edge
        lda PLAYERX
        cmp #XMAX+1
        bcs RightColl
	; Check Upper Square
	adc #7
	tax
	sec
	lda PLAYERY
	sbc #3
	jsr MapCords
	; Wall Collision
	cmp #1
	beq RightColl
	; Check Lower Square
	clc
	lda PLAYERX
	adc #7
	tax
	sec
	lda PLAYERY
	sbc #8
	jsr MapCords
	; Wall Collision
	cmp #1
	bne EndRight
RightColl
	lda COLLIDE
	and #%01111111
	sta COLLIDE
EndRight

	; Check Player LHS
CheckLeft	
        ; Check Screen Edge
        ldx PLAYERX
        cpx #XMIN-1
        bcc LeftColl
	; Check Upper Square
	sec
	lda PLAYERY
	sbc #3
	jsr MapCords
	cmp #1
	beq LeftColl
	; Check Lower Square
	ldx PLAYERX
	sec
	lda PLAYERY
	sbc #8
	jsr MapCords
	cmp #1
	bne EndLeft
LeftColl
	lda COLLIDE
	and #%10111111
	sta COLLIDE
EndLeft		

CheckDown
	; Check Screen Bottom
	lda PLAYERY
	cmp #PHEIGHT
	bcc DownColl
	; Check LHS Square
	sbc #PHEIGHT-2
	ldx PLAYERX
	inx
	jsr MapCords
	cmp #1
	beq DownColl
	cmp #0
	beq CheckDown2
	lda COLLIDE
	and #%11011011
	ora #%00100000
	sta COLLIDE
CheckDown2
	; Check RHS Square
	clc
	lda PLAYERX
	adc #6
	tax
	sec
	lda PLAYERY
	sbc #PHEIGHT-2
	jsr MapCords
	cmp #1
	beq DownColl
	cmp #0
	beq EndDown
	lda COLLIDE
	and #%11011011	
	ora #%00100000
	sta COLLIDE
	bne EndDown
DownColl
	lda COLLIDE
	and #%11011011
	sta COLLIDE
EndDown	

CheckUp
	; Check LHS Square
	clc
	lda PLAYERX
	adc #6
	tax
	; Check Screen Edge
	lda PLAYERY
	cmp #KERNH
	bcs UpColl
	sec
	sbc #2
	jsr MapCords
	cmp #1
	beq UpColl
	cmp #2
	bne CheckUp2
	lda COLLIDE
	ora #%00010000
	sta COLLIDE
CheckUp2
	; Check RHS Square
	ldx PLAYERX
	inx
	sec
	lda PLAYERY
	sbc #2
	jsr MapCords
	cmp #1
	beq UpColl
	cmp #2
	bne EndUp
	lda COLLIDE
	ora #%00010000
	sta COLLIDE	
	bne EndUp
UpColl
	lda COLLIDE
	and #%11100111
	sta COLLIDE
EndUp

	; Check Bell Collisions
	lda CXM0P
	and #%01000000
	beq EndBell
	; Calculate Player Width
	clc
	lda PLAYERX
	adc #6
	sta PEND
	; Check First Player Row
CheckBell1
	; Calculate Upper Row (PLAYERY-3/8)
	sec
	lda PLAYERY
	sbc #1
	lsr
	lsr
	lsr
	tay			; ROW
	; Fetch Bell Data For Row
	lda BELL,Y
	; Move to Next Row If No Bell
	beq BellColl2
	; Convert Position To Player Coordinates (POS + 7)
	clc
	adc #7
	sta PSWITCH		; BELLX
Check1
	; Check If BELLX Is Between PLAYERX and PLAYERX+5
	cmp PLAYERX
	bmi Check2
	cmp PEND
	bmi GotBell
Check2
	; Calculate Bell Width	
	clc
	adc #8
	sta PEND		; BELLX+8
	; Check If PLAYERX Is Between BELLX and BELLX+7
	lda PLAYERX
	cmp PSWITCH
	bmi BellColl2
	cmp PEND
	bmi GotBell
BellColl2
	; Check Lower Row (PLAYERY-8/8)
	sec
	lda PLAYERY
	sbc #8
	lsr
	lsr
	lsr
	tay			; ROW
GotBell
	lda BELL,Y		; Paranoia
        beq EndBell
	; Play Sound
	lda #12
	sta AUDV0 
	sta AUDC0
	lda #2
	sta AUDF0
	sta DUR0 	
	; Remove Bell
	lda #0
	sta BELL,Y
	dec BELLS
EndBell

EndPlayerColl
	
	; Construct Knight Collision Vector KMOVE
	; Bit 7 - Can Move Right
	; Bit 6 - Can Move Left
	; Bit 5 - Can Move Down Ladder
	; Bit 4 - Can Move Up Ladder
	; Bit 3 - Fall
	; Bit 2 - Show Esmeralda
	; Bit 1-0 Movement Type (00=Static, 01=Follow, 10=Thrusting)

	; Set Defaults
	lda KMOVE
	and #%00000111
	ora #%11111000
	sta KMOVE
			
	; Check Knight RHS
CheckKRight
	; Check Screen Edge
	lda KNIGHTX
        cmp #XMAX
        bcs KRightColl
        ; Check Upper Square
        adc #8
        tax
	sec
        lda KNIGHTY
	sbc #3
        jsr MapCords
        ; Wall Collision
        cmp #1
        beq KRightColl
        ; Check Lower Square
        clc
        lda KNIGHTX
        adc #8
        tax
        sec
        lda KNIGHTY
        sbc #10
        jsr MapCords
        ; Wall Collision
        cmp #1
	bne EndKRight
KRightColl
	lda KMOVE
	and #%01111111
	sta KMOVE
EndKRight
	
	; Check Knight LHS
CheckKLeft	
        ; Check Screen Edge
        ldx KNIGHTX
        cpx #XMIN
        bcc KLeftColl
	; Check Upper Square
	dex
	sec
	lda KNIGHTY
	sbc #3
	jsr MapCords
	cmp #1
	beq KLeftColl
	; Check Lower Square
	ldx KNIGHTX
	dex
	sec
	lda KNIGHTY
	sbc #10
	jsr MapCords
	cmp #1
	bne EndKLeft
KLeftColl
	lda KMOVE
	and #%10111111
	sta KMOVE
EndKLeft

CheckKDown
	; Check Screen Bottom
	lda KNIGHTY
	cmp #KHEIGHT+2
	bcc KDownColl
	; Check LHS Square
	sbc #11
	ldx KNIGHTX
	jsr MapCords
	cmp #1
	beq KDownColl
	cmp #2
	bne CheckKDown2
	lda KMOVE
	and #%11110111
	sta KMOVE
CheckKDown2
	; Check RHS Square
	clc
	lda KNIGHTX
	adc #7
	tax
	sec
	lda KNIGHTY
	sbc #11
	jsr MapCords
	cmp #1
	beq KDownColl
	cmp #2
	bne EndKDown
	lda KMOVE
	and #%11110111
	sta KMOVE
	jmp EndKDown
KDownColl
	lda KMOVE
	and #%11010111
	sta KMOVE
EndKDown

CheckKUp
	; Check LHS Square	
	clc
	lda KNIGHTX
	adc #7
	tax	
	; Check Screen Edge
	lda KNIGHTY
	cmp #KERNH
	bcs KUpColl
	sec
	sbc #2
	jsr MapCords
	cmp #2
	bne KUpColl
	; Check RHS Square
	ldx KNIGHTX
	sec
	lda KNIGHTY
	sbc #2
	jsr MapCords
	cmp #2
	beq EndKUp
KUpColl
	lda KMOVE
	and #%11101111
	sta KMOVE	
EndKUp
	
        ; Finish Over-scan
WaitOverscanEnd
        lda INTIM
        bne WaitOverscanEnd
        
        ; Loop To Beginning
        jmp MainLoop

; -----------------------------------------------------------------------------
; Game Subroutines
; -----------------------------------------------------------------------------

	; MapCords Subroutine
	; Translates Coordinates (X,Y) to Map
	; On Entry - X,A = Coordinates
	; On Exit - A = Map Data 
MapCords
	; Calculate Row Y/8 (0-11)
	lsr			; [0] + 2
	and #%11111100		; [2] + 2
	sta ROW			; [4] + 3	ROW
	; Calculate Column (X-XMIN)/8 (0-15)
	sec			; [7] + 2
	txa			; [9] + 2
	sbc #XMIN-1		; [11] + 2
	; Bounds Check (X<0)
	bmi NoCheck		; [13] + 2
	lsr			; [15] + 2
	lsr			; [17] + 2
	lsr			; [19] + 2
	tax			; [21] + 2	COL
	; Calculate Block COL/4 (0-3)
	lsr			; [23] + 2
	lsr			; [25] + 2	BLOCK
	; Calculate Screen Index (ROW*4) + BLOCK
	clc			; [27] + 2
	adc ROW			; [29] + 3
	tay			; [32] + 2	INDEX
	; Make Pointer and Fetch Map Data
	lda SCREEN,Y		; [34] + 4	Block LSB
	sta PPTR		; [38] + 3
	lda #>Tile0		; [41] + 2	Block MSB	
	sta PPTR+1		; [43] + 3
	ldy #0			; [46] + 2
	lda (PPTR),Y		; [48] + 5
	; Shift Map Data
	ldy Shifts,X		; [53] + 3	SHIFT
	beq EndShift		; [56] + 2/3
ShiftLoop			
	lsr			; [58] + 2
	lsr			; [60] + 2
	dey			; [62] + 2
	bne ShiftLoop		; [64] + 2/3
EndShift
	; Mask Extra Bits
	and #%00000011		; [66] + 2
NoCheck
	rts	
						
	; Reset Routines
ResetGame
	; Clear Bit 7 & Wrap at Level 16
	lda LEVEL
	and #%00001111
	
	; Check If LIVES = 0 (Game Over)
	ldx LIVES
	bne NoEnd
	; Reset Level, Bells, Score & Lives
	txa
	sta BELLS
	sta SCORE
	sta SCORE+1
	ldx #STARTLIVES
	stx LIVES
NoEnd
	; Store Level
	sta LEVEL
	
	; Fetch Level Data Pointer
	asl
	tax
	lda LevelPtr,X		; Load LSB
	sta PPTR
	lda LevelPtr+1,X	; Load MSB
	sta PPTR+1

	; Reset Various Variables
	ldy #0	
	sty CYCLE
	sty COLLIDE
	sty JSTEP
	sty MCOUNT
 	sty NVOL
 	sty NOTE
	sty NTIME
	inc NTIME
	
	; Reset Missile Positions
	ldx #MAXROW-1
ResetMissileLoop
	sty MISSILE,X
	dex
	bpl ResetMissileLoop	

	; Copy Level Data Into Variables
InitLoop
	lda (PPTR),Y
	ldx VarList,Y
	beq EndInit
	sta $00,X
	iny
	bne InitLoop
EndInit
		
	; Check If Bells = 0 (Reset Level Data)
	lda BELLS
	bne EndBellCopy
	; Read Bell Data
	inx
BellLoop
	; Fetch Bell Position
	lda (PPTR),Y
	asl
	asl
	asl
	sta BELL,X
	beq NextBell
	; Increment Bell Count
	inc BELLS
NextBell
	inx
	iny
	cpx #MAXROW
	bcc BellLoop	
	
	; Reset Timer (Set To 0 If On Level 0 or 15)
	ldx #0
	lda LEVEL
	and #%00001111
 	beq NoTimer
 	cmp #15
 	beq NoTimer
 	ldx #31
NoTimer
 	stx TIMER
EndBellCopy

	; Reset Random Seed
	ldy #15
	sty RANDOM	
		
	; Copy Screen Data
	lda LEVEL
	cmp #15
	beq MakeFinale
	lda #>ScreenData
	sta KPTR+1
	ldy #0
	ldx #0
	clc
ScreenCopyLoop
	lda (KPTR),Y
	and #%00001111
	asl
	asl
	asl
	adc #<Tile0
	sta SCREEN,X
	inx
	lda (KPTR),Y
	and #%11110000
	lsr
	adc #<Tile0	
	sta SCREEN,X
	inx
	iny
	cpx #(MAXROW*4)
	bcc ScreenCopyLoop	
	rts
MakeFinale
	ldx #47
	lda #<Tile0
ClsLoop
	sta SCREEN,X
	dex
	bpl ClsLoop
	lda #<Tile11
	sta SCREEN
	sta SCREEN+1
	sta SCREEN+3
	lda #<TileQ+4
	sta SCREEN+2
 	lda #<Tile0-1
 	sta SCREEN+4
 	sta SCREEN+6
	rts
		
DisplayScore
	; Set Font Pointers (MSB)
	lda #>Font
	ldy #10
FontLoop
	sta SPTR0+1,Y
	dey
	dey
	bpl FontLoop
	
	; Display Level or Game Over
	lda LIVES
	bne LevelNumber
GameOver
	; Copy Game Over Message
 	lda #<Space
 	sta SPTR0
	sta KPTR
 	lda #<GA
 	sta SPTR1
 	lda #<ME
 	sta SPTR2
 	lda #<OV
 	sta SPTR3
 	lda #<ER
 	sta PPTR
	; Set Background Colour
	ldx #RED
	bne EndLevelNumber
LevelNumber
	; Copy Level: Message
 	lda #<LE
 	sta SPTR0
 	lda #<VE
 	sta SPTR1
 	lda #<L
 	sta SPTR2
 	lda #<Space
 	sta SPTR3
	
	; Convert LEVEL to BCD
	lda LEVEL
	and #%00001111	
 	sta ROW	
 	lda #0
 	sta END
 	ldx #8
 	sed
ConvertLoop
 	asl ROW
 	lda END
 	adc END
 	sta END
 	dex
 	bne ConvertLoop
 	cld
	
	; Convert BCD To Text
 	and #%11110000
 	lsr
 	lsr
 	lsr
 	lsr
 	tax
 	lda Digit,X
 	sta PPTR
 	lda END
 	and #%00001111
 	tax
 	lda Digit,X
 	sta KPTR
	; Set Background Colour
	ldx #BLUE
EndLevelNumber

	; Set PF Colour and Reflected PF
	stx COLUPF	
	lda #%00000001
	sta CTRLPF
		
	; Wait for Vertical Blank End
WaitVblankEnd
        lda INTIM
        bne WaitVblankEnd
        sta VBLANK
	sta WSYNC
	; Silence All Sounds
	sta AUDV0
	sta AUDV1
	
	; Set Playfield Configuration
	sta PF0
	lda #%11111111
	sta PF1
	sta PF2
		
	; Display Text
	jsr ScoreKernel
		
	; Calculate Score Pointers
	lda #3
	sta END
DigitLoop
	lda END
	; SPTR Index = END * 2
	asl
	tay
	; SCORE Index = END / 2
	lsr
	lsr
	tax
	lda SCORE,X
	bcs Lower
Upper
	; Obtain Upper Digit Index
	and #%11110000
	lsr
	lsr
	lsr
	lsr
	BITABS
Lower
	; Obtain Lower Digit Index
	and #%00001111
	; Store Sprite Pointer	
	tax
	lda Digit,X
	sta SPTR1,Y
	; Next Digit
	dec END	
	bpl DigitLoop

	; Display Space in Remaining Positions
	lda #<Space
	sta SPTR0
 	sta KPTR	
	
	; Display Score
	jsr ScoreKernel

	; Reset PF (A is 0 After ScoreKernel)
	sta WSYNC
	sta PF1
	sta PF2
	
	; Pad Out Screen with Blank Lines
	IF PALTIME
	ldx #201
	ELSE
	ldx #166
	ENDIF
BlankLoop
	sta WSYNC	
	dex
	bne BlankLoop

        ; Start Vertical Blank
        lda #2
        sta WSYNC
        sta VBLANK
	
        ; Set Timer for Over-scan
        IF PALTIME
        ldy #42                 ; (36*76)/64 = 42
        ELSE
        ldy #35                 ; (30*76)/64 = 35
        ENDIF
        sty TIM64T

	; Read Fire Button (After Delay)
	lda FDELAY
	beq CheckFire
	dec FDELAY
	bne EndFire
CheckFire
	lda INPT4
	bmi EndFire
	jsr ResetGame
EndFire
	
	; Wait For End Of Over-scan
	jmp WaitOverscanEnd	

PlaySequence
        dec NTIME
        bmi NextNote
	jmp NotNewNote
NextNote
        stx NTIME	
	cmp #15
	bne PlayNote
	; Display Ending Sequence (Synchronised To Music)
        lda NOTE
	IF PALTIME
        cmp #93
	ELSE
        cmp #92
	ENDIF
	; Start Hunchy Moving
        bne NotEFHunchGo
        ldx #<Tile0
        stx SCREEN+4
NotEFHunchGo
	; Raise Bridge
        cmp #71
        beq YesEFBridge
        cmp #65
        beq YesEFBridge
        cmp #59
        beq YesEFBridge
        cmp #53
        bne NotEFBridge
YesEFBridge
        ; Scroll Bridge (By Decreasing Address)
        dec SCREEN+2
NotEFBridge
        ; Reset Tile Above Platform (To Allow Esmeralda To Move Forward)
        cmp #47
        bne NotEFEsmerelda
        ldx #<Tile0
        stx SCREEN+6
	; Prevent Esmeralda Moving Onto Hunchy
        dex
        stx SCREEN+5
NotEFEsmerelda
        cmp #28
        bcc NotEFHeart
        cmp #36
        bcs NotEFHeart
        ; Clear Lever
	lda #0
 	sta BELL+2
        ; Start Missiles
        lda #%11100111
        sta MDIR
        ldx #27
EF3lp
        ; Display Scrolling Heart
        lda SCREEN+12,X
        sta SCREEN+16,X
        dex
        bpl EF3lp
        lda NOTE
        and #7
        tax
        lda Heart,X
        sta SCREEN+13
        sta SCREEN+14
NotEFHeart
	; Finish At End Of Tune
        lda NOTE
        cmp #23
        bne NotEFLeft
        ldx #<Tile0
        stx SCREEN+5
NotEFLeft	
	cmp #1
        bne PlayNote
        dec BELLS    	   
PlayNote
	; Move Note Pointer
        dec NOTE
        bpl NoteOK1
	; Reset Tune
        lda #TUNELENGTH
        sta NOTE
NoteOK1
	; Fetch Note
        lda NOTE
        lsr
        tax
        lda NoteData,X
        bcs NoteOK2
        lsr
        lsr
        lsr
        lsr
NoteOK2
        and #15
        beq NotNewNote
	; Play Note
        tax
        lda PitchTbl,X
        sta AUDF1
        lda #26
        sta NVOL
NotNewNote
	; Fade Each Note
        lda NVOL
        beq NoteBad2
        dec NVOL
        lsr
        sta AUDV1
NoteBad2
	rts
	           		
; -----------------------------------------------------------------------------
; Game Data
; -----------------------------------------------------------------------------
	
	; Game Music (Teddybears Picnic)
NoteData
	DC.B	$00,$00,$67,$89,$AB,$54,$0B,$9B,$96,$0B,$60,$9B
	DC.B	$09,$69,$B6,$05,$60,$56,$04,$23,$42,$01,$20,$12
	DC.B	$00,$00,$64,$05,$45,$64,$05,$40,$65,$45,$40,$6B
	DC.B	$00,$00,$96,$07,$67,$96,$07,$60,$97,$67,$60,$97
	
	ORG	$FB00
	
	; Music Pitch Table
PitchTbl = *-1
	DC.B	$0D,$0F,$11,$12,$14,$17,$18,$1A,$1B,$1D,$1F
		
	; Level Data (Part 1)
Title
	DC.B	<TitleScreen			; Screen  
	DC.B	BLUE				; Tile Colour	
	DC.B	%00000111			; Missile Max,Delay,Dir 
	DC.B	%00100000,%00100000,30		; Knight Swap Size 
	DC.B	32,16				; Player Start X,Y 
	DC.B	54,50				; Knight Position X,Y
	DC.B	%00000101			; Knight Pattern
	DC.B	8,0,0,0,0,0,0,0,0,0,0		; Bell Positions
Level0
        DC.B    <Screen0			; Screen 
        DC.B    DARKGREEN			; Tile Colour
        DC.B    %00111101                       ; Missile Max,Delay,Dir
        DC.B    %00100000,%00000000,0           ; Knight Swap Size 
        DC.B    136,96                          ; Player Start X,Y 
        DC.B    20,0				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	16,1,14,0,0,16,10,0,13,2,7	; Bell Positions	
Level1
	DC.B	<Screen1			; Screen 
	DC.B	ORANGE				; Tile Colour	
	DC.B	%01011101			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size 
	DC.B	136,56				; Player Start X,Y 
	DC.B	20,0				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	0,13,1,7,13,0,7,1,0,15,7	; Bell Positions
Level2
	DC.B	<Screen2			; Screen 
	DC.B	GREY				; Tile Colour
	DC.B	%01010101			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000010,60		; Knight Swap Size 
	DC.B	136,16				; Player Start X,Y 
	DC.B	73,50				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	7,5,1,10,4,10,14,1,0,8,0	; Bell Positions
Level3
	DC.B	<Screen3			; Screen 
	DC.B	DARKBLUE			; Tile Colour
	DC.B	%00111111			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size 
	DC.B	96,32				; Player Start X,Y 
	DC.B	20,0				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	1,5,16,9,0,16,5,12,9,1,11	; Bell Positions
Level4
	DC.B	<Screen4			; Screen 
	DC.B	BROWN				; Tile Colour
	DC.B	%01010110			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size  
	DC.B	24,16				; Player Start X,Y 
	DC.B	49,66				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	16,0,2,9,7,4,1,0,4,14,7		; Bell Positions
Level5
	DC.B	<Screen5			; Screen 
	DC.B	MAGENTA				; Tile Colour
	DC.B	%01011111			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size 
	DC.B	64,88				; Player Start X,Y 
	DC.B	88,38				; Knight Start X,Y
	DC.B	%00000010			; Knight Pattern
	DC.B	2,14,5,9,1,12,10,1,9,16,11	; Bell Positions	
Level6
	DC.B	<Screen0			; Screen 
	DC.B	DARKGREEN			; Tile Colour
	DC.B	%01111111			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size 
	DC.B	40,40				; Player Start X,Y 
	DC.B	50,18				; Knight Start X,Y
	DC.B	%00000001			; Knight Pattern
	DC.B	1,15,16,0,0,14,4,11,15,2,11	; Bell Positions
Level7
	DC.B	<Screen1			; Screen 
	DC.B	ORANGE				; Tile Colour
	DC.B	%10010101			; Missile Max,Delay,Dir
	DC.B	%00100000,%00100000,50		; Knight Swap Size 
	DC.B	50,32				; Player Start X,Y 
	DC.B	50,18				; Knight Start X,Y
	DC.B	%00000001			; Knight Pattern
	DC.B	9,1,9,16,0,9,7,14,0,1,16	; Bell Positions
Level8
	DC.B	<Screen2			; Screen 
	DC.B	GREY				; Tile Colour
	DC.B	%10110011			; Missile Max,Delay,Dir
	DC.B	%00010000,%00000010,80		; Knight Swap Size 
	DC.B	136,16				; Player Start X,Y 
	DC.B	72,54				; Knight Start X,Y
	DC.B	%00000010			; Knight Pattern
	DC.B	0,6,0,7,16,1,10,3,0,9,16	; Bell Positions	
Level9
	DC.B	<Screen3			; Screen 
	DC.B	DARKBLUE			; Tile Colour
	DC.B	%10011101			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size 
	DC.B	96,16				; Player Start X,Y 
	DC.B	56,70				; Knight Start X,Y
	DC.B	%00000010			; Knight Pattern
	DC.B	7,9,11,4,9,1,13,9,1,10,16	; Bell Positions
		
	ORG	$FC00

Level10
	DC.B	<Screen4			; Screen 
	DC.B	BROWN				; Tile Colour
	DC.B	%11010111			; Missile Max,Delay,Dir
	DC.B	%00100000,%00000000,0		; Knight Swap Size  
	DC.B	40,16				; Player Start X,Y 
	DC.B	84,66				; Knight Start X,Y
	DC.B	%00000001			; Knight Pattern
	DC.B	1,13,4,7,9,1,14,16,8,11,1	; Bell Positions
	
ScreenData
	; Screen Layout Data	
TitleScreen
	DC.B	11|(11<<4),	1|(11<<4),	0|(0<<4),	0|(7<<4)
	DC.B	0|(0<<4),	0|(7<<4),	0|(0<<4),	9|(11<<4)
	DC.B	0|(5<<4),	2|(0<<4),	1|(4<<4),	2|(3<<4)
	DC.B	1|(4<<4),	9|(3<<4),	1|(4<<4),	7|(5<<4)
	DC.B	7|(4<<4),	7|(3<<4),	9|(4<<4),	6|(3<<4)
	DC.B	7|(6<<4),	4|(7<<4),	4|(0<<4),	4|(7<<4)
Screen0
 	DC.B	11|(11<<4),	11|(11<<4),	0|(15<<4),	0|(0<<4)
 	DC.B	0|(15<<4),	0|(2<<4),	9|(13<<4),	11|(7<<4)
 	DC.B	0|(15<<4),	0|(0<<4),	0|(15<<4),	0|(2<<4)
 	DC.B	5|(13<<4),	4|(0<<4),	0|(15<<4),	5|(0<<4)
 	DC.B	0|(15<<4),	0|(7<<4),	4|(13<<4),	10|(0<<4)
 	DC.B	0|(15<<4),	0|(2<<4),	0|(0<<4),	0|(0<<4)
Screen1
	DC.B	11|(11<<4),	11|(11<<4),	14|(15<<4),	14|(15<<4)
	DC.B	14|(13<<4),	12|(15<<4),	14|(15<<4),	14|(15<<4)
	DC.B	14|(0<<4),	0|(0<<4),	12|(11<<4),	11|(6<<4)
	DC.B	14|(15<<4),	14|(0<<4),	14|(0<<4),	14|(3<<4)
	DC.B	14|(15<<4),	14|(7<<4),	12|(13<<4),	12|(7<<4)
	DC.B	14|(15<<4),	14|(2<<4),	0|(0<<4),	0|(0<<4)
Screen2
	DC.B	8|(7<<4),	5|(2<<4),	14|(0<<4),	0|(9<<4)
	DC.B	14|(0<<4),	5|(0<<4),	14|(7<<4),	0|(0<<4)
	DC.B	14|(2<<4),	1|(2<<4),	14|(9<<4),	10|(9<<4)
	DC.B	0|(0<<4),	0|(15<<4),	8|(0<<4),	0|(15<<4)
	DC.B	0|(0<<4),	0|(15<<4),	9|(11<<4),	11|(13<<4)
	DC.B	0|(0<<4),	0|(0<<4),	0|(0<<4),	0|(0<<4)
Screen3
	DC.B	8|(7<<4),	9|(7<<4),	14|(0<<4),	4|(0<<4)
	DC.B	12|(6<<4),	9|(13<<4),	14|(0<<4),	4|(15<<4)
	DC.B	0|(11<<4),	9|(13<<4),	12|(0<<4),	4|(15<<4)
	DC.B	1|(3<<4),	1|(0<<4),	12|(0<<4),	12|(0<<4)
	DC.B	14|(7<<4),	4|(13<<4),	1|(0<<4),	4|(15<<4)
	DC.B	14|(10<<4),	8|(0<<4),	8|(0<<4),	1|(7<<4)
Screen4
	DC.B	11|(11<<4),	11|(11<<4),	0|(0<<4),	0|(15<<4)
	DC.B	0|(0<<4),	0|(15<<4),	8|(11<<4),	11|(13<<4)
	DC.B	14|(2<<4),	14|(15<<4),	14|(2<<4),	14|(0<<4)
	DC.B	0|(11<<4),	12|(7<<4),	0|(15<<4),	14|(0<<4)
	DC.B	1|(15<<4),	1|(9<<4),	14|(15<<4),	10|(3<<4)
	DC.B	12|(15<<4),	4|(0<<4),	0|(0<<4),	0|(0<<4)
Screen5
	DC.B	4|(9<<4),	9|(0<<4),	1|(0<<4),	0|(3<<4)
	DC.B	14|(2<<4),	10|(0<<4),	12|(0<<4),	8|(0<<4)
	DC.B	14|(3<<4),	0|(15<<4),	14|(11<<4),	0|(13<<4)
	DC.B	1|(0<<4),	10|(0<<4),	0|(15<<4),	5|(0<<4)
	DC.B	4|(15<<4),	1|(15<<4),	1|(13<<4),	0|(13<<4)
	DC.B	8|(0<<4),	10|(3<<4),	0|(0<<4),	1|(0<<4)
Screen6
	DC.B	1|(10<<4),	4|(3<<4),	0|(7<<4),	8|(0<<4)
	DC.B	14|(0<<4),	0|(0<<4),	14|(0<<4),	14|(0<<4)
	DC.B	14|(6<<4),	12|(3<<4),	12|(0<<4),	0|(2<<4)
	DC.B	0|(10<<4),	11|(3<<4),	4|(3<<4),	14|(0<<4)
	DC.B	4|(3<<4),	0|(2<<4),	4|(11<<4),	12|(0<<4)
	DC.B	0|(0<<4),	0|(0<<4),	0|(7<<4),	8|(2<<4)
Screen7
	DC.B	8|(11<<4),	0|(3<<4),	0|(15<<4),	0|(0<<4)
	DC.B	1|(0<<4),	0|(15<<4),	14|(2<<4),	14|(15<<4)
	DC.B	4|(0<<4),	0|(0<<4),	14|(0<<4),	14|(0<<4)
	DC.B	1|(15<<4),	0|(10<<4),	0|(0<<4),	14|(3<<4)
	DC.B	4|(0<<4),	0|(3<<4),	0|(15<<4),	14|(0<<4)
	DC.B	1|(13<<4),	12|(0<<4),	0|(0<<4),	0|(0<<4)

	; Lives PF Data
Lives1	DC.B	%00000000	; 0 Life
	DC.B	%01000000	; 1 Life
	DC.B	%01010000	; 2 Lives
	DC.B	%01010100	; 3 Lives
	DC.B	%01010101	; 4 Lives
	DC.B	%01010101	; 5 Lives
	DC.B	%01010101	; 6 Lives
	DC.B	%01010101	; 7 Lives		
Lives2
	DC.B	%00000000	; 0
	DC.B	%00000000	; 1
	DC.B	%00000000	; 2
	DC.B	%00000000	; 3
	DC.B	%00000000	; 4	
	DC.B	%00000010	; 5
	DC.B	%00001010	; 6
	DC.B	%00101010	; 7
		
	ORG	$FD00

	; Level Data (Part 2)			
Level11
	DC.B	<Screen5			; Screen 
	DC.B	MAGENTA				; Tile Colour
	DC.B	%01111010			; Missile Max,Delay,Dir
	DC.B	%00100000,%00100000,72		; Knight Swap Size 
	DC.B	64,16				; Player Start X,Y 
	DC.B	20,0				; Knight Start X,Y
	DC.B	%00000000			; Knight Pattern
	DC.B	10,1,16,7,11,0,1,13,2,16,11	; Bell Positions
Level12
	DC.B	<Screen6			; Screen 
	DC.B	CYANGREEN			; Tile Colour
	DC.B	%01111101			; Missile Max,Delay,Dir 
	DC.B	%00100000,%00100110,28		; Knight Swap Size 
	DC.B	40,56				; Player Start X,Y 
	DC.B	56,22				; Knight Start X,Y
	DC.B	%00000010			; Knight Pattern
	DC.B	16,9,1,13,7,16,7,5,16,9,1	; Bell Positions	
Level13
	DC.B	<Screen7			; Screen 
	DC.B	PURPLE				; Tile Colour	
	DC.B	%10010110			; Missile Max,Delay,Dir
	DC.B	%00010000,%00000000,0		; Knight Swap Size 
	DC.B	40,32				; Player Start X,Y 
	DC.B	72,46				; Knight Start X,Y
	DC.B	%00000010			; Knight Pattern
	DC.B	15,1,9,11,16,9,13,1,9,16,6	; Bell Positions
Final
	DC.B	<TitleScreen			; Screen  
	DC.B	LIGHTRED				; Tile Colour	
	DC.B	%00000000			; Missile Max,Delay,Dir
	DC.B	%00100000,%00100000,30		; Knight Swap Size 
	DC.B	18,16				; Player Start X,Y 
	DC.B	128,18				; Knight Start X,Y
	DC.B	%00000101			; Knight Pattern
	DC.B	0,8,0,0,0,0,0,0,0,0,0		; Bell Positions

	; Heart Data (Final Level)
Heart
	DC.B	<Tile11,<Tile11,<Tile11,<Tile16,<Tile0,<Tile2,<Tile6,<Tile10
	
        ORG	($FD00 + KERNH)
	
	; Tile Data For Screen - 8 Bytes:	
	; Byte 0 - Map Data (00=Empty, 01=Wall, 10=Ladder, 11=Reserved)
	; Bytes 1-7 - Tile Data
TileQ				; Bridge
	DC.B    %01010101
        DC.B    %00000000
        DC.B    %00000000
        DC.B    %00000000
        DC.B    %00000000
        DC.B    %00000000
        DC.B    %10101010
        DC.B    %01010101	
Tile0				; 0000
	DC.B	%00000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
Tile1				; 0001
	DC.B	%00000001
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000011
	DC.B	%00000010
	DC.B	%00000001
	DC.B	%00000011
Tile2				; 1000
	DC.B	%01000000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11000000
	DC.B	%01000000
	DC.B	%10000000
	DC.B	%11000000
Tile3				; 0010
	DC.B	%00000100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001100
	DC.B	%00000100		
	DC.B	%00001000
	DC.B	%00001100
Tile4				; 0100
	DC.B	%00010000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110000
	DC.B	%00100000
	DC.B	%00010000
	DC.B	%00110000
Tile5				; 0011
	DC.B	%00000101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001111
	DC.B	%00000110
	DC.B	%00001001
	DC.B	%00001111
Tile6				; 1100
	DC.B	%01010000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11110000
	DC.B	%01100000		
	DC.B	%10010000
	DC.B	%11110000
Tile7				; 0101
	DC.B	%00010001
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110011
	DC.B	%00100010		
	DC.B	%00010001
	DC.B	%00110011	
Tile8				; 1010
	DC.B	%01000100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11001100
	DC.B	%01000100
	DC.B	%10001000
	DC.B	%11001100
Tile9				; 0111
	DC.B	%00010101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00111111
	DC.B	%00100110
	DC.B	%00011001
	DC.B	%00111111
Tile10				; 1110
	DC.B	%01010100
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111100
	DC.B	%01100100		
	DC.B	%10011000
	DC.B	%11111100
Tile11				; 1111
	DC.B	%01010101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%11111111
	DC.B	%01100110		
	DC.B	%10011001
	DC.B	%11111111
Tile12				; 11L1
	DC.B	%01011001
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00001100
	DC.B	%11110011
	DC.B	%01100010
	DC.B	%10010001
	DC.B	%11111111
Tile13				; 1L11
	DC.B	%01100101
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00110000
	DC.B	%11001111
	DC.B	%01000110		
	DC.B	%10001001
	DC.B	%11111111
Tile14				; 00L0
	DC.B	%00001000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00001100
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00001100
Tile15				; 0L00
	DC.B	%00100000
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00110000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00110000
Tile16				; 0110
        DC.B    %00010100
        DC.B    %00000000       
        DC.B    %00000000
        DC.B    %00000000
        DC.B    %00111100
        DC.B    %00100100
        DC.B    %00011000
        DC.B    %00111100	
		
	; Time Bar PF Data (Lower)
TimeBar1						
	DC.B	%00000000	; 0
	DC.B	%01000000	; 1
	DC.B	%01100000	; 2
	DC.B	%01110000	; 3
	DC.B	%01111000	; 4
	DC.B	%01111100	; 5
	DC.B	%01111110	; 6
	DC.B	%01111111	; 7
	DC.B	%01111111	; 8
	DC.B	%01111111	; 9
	DC.B	%01111111	; 10
	DC.B	%01111111	; 11
	DC.B	%01111111	; 12
	DC.B	%01111111	; 13
	DC.B	%01111111	; 14
	DC.B	%01111111	; 15

	ORG	$FE00
	
	; Time Bar PF Data (Upper)
TimeBar2
	DC.B	%00000000	; 0
	DC.B	%00000000	; 1
	DC.B	%00000000	; 2
	DC.B	%00000000	; 3
	DC.B	%00000000	; 4
	DC.B	%00000000	; 5
	DC.B	%00000000	; 6
	DC.B	%00000000	; 7
	DC.B	%00000000	; 8
	DC.B	%00000001	; 9
	DC.B	%00000011	; 10
	DC.B	%00000111	; 11
	DC.B	%00001111	; 12
	DC.B	%00011111	; 13
	DC.B	%00111111	; 14
	DC.B	%01111111	; 15
	
	; Level Data Pointers
LevelPtr
	DC.B	<Title,>Title
	DC.B	<Level0,>Level0
	DC.B	<Level1,>Level1
	DC.B	<Level2,>Level2
	DC.B	<Level3,>Level3
	DC.B	<Level4,>Level4
	DC.B	<Level5,>Level5
	DC.B	<Level6,>Level6	
	DC.B	<Level7,>Level7	
	DC.B	<Level8,>Level8
	DC.B	<Level9,>Level9	
	DC.B	<Level10,>Level10	
	DC.B	<Level11,>Level11
	DC.B	<Level12,>Level12
	DC.B	<Level13,>Level13	
	DC.B	<Final,>Final
	
	; Collision Detection Shifts (For Map Data)	
Shifts
	DC.B	3,2,1,0,0,1,2,3,3,2,1,0,0,1,2,3	

	; Bell Frames
BellFrames
	DC.B	<Bell1,<Bell2,<Bell3,<Bell4,<Bell4,<Bell3,<Bell2,<Bell1	
	
	; Missile Shapes
MissileShapes
	DC.B	<Missile3,<Missile1,<Missile2,<Missile1
	DC.B	<Missile2,<Missile1,<Missile2,<Missile3

	; Variables Initialised on Level Reset (Final 0 Is Vital!)
VarList
	DC.B	KPTR,COLUPF,MDIR,KNUSIZ0,KNUSIZ1,KRESIZE
	DC.B	PLAYERX,PLAYERY,KNIGHTX,KNIGHTY,KMOVE,0	
		
	ORG	($FE00 + KERNH)

	; Sprite Data 1
StartSprites1
	; Player Sprites
Player0
	DC.B	%00000000
	DC.B	%00011100
        DC.B	%00011000
        DC.B	%00011100
        DC.B	%00110110
        DC.B	%01101110
        DC.B	%01101110
        DC.B	%00111100
        DC.B	%00010010
        DC.B	%00111010
        DC.B	%00011100
Player1
        DC.B	%00000000
        DC.B	%00100100
        DC.B	%01001110
        DC.B	%00011100
        DC.B	%00011110
        DC.B	%01011110
        DC.B	%01101110
        DC.B	%00111100
        DC.B	%00010010
        DC.B	%00111010
        DC.B	%00011100
Player2
        DC.B	%00000000
        DC.B	%00100100
        DC.B	%01110010
        DC.B	%00111100
        DC.B	%00111110
        DC.B	%01110010
        DC.B	%01101110
        DC.B	%00111100
        DC.B	%00010010
        DC.B	%00111010
        DC.B	%00011100
Player3
        DC.B	%00000000
        DC.B	%00110000
        DC.B	%00110110
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111100
        DC.B	%00111110
        DC.B	%01111010
        DC.B	%01100110
        DC.B	%01100100
        DC.B	%01011000
Player4
        DC.B	%00000000
        DC.B	%00001100
        DC.B	%01101100
        DC.B	%01111100
        DC.B	%01111100
        DC.B	%00111100
        DC.B	%01111100
        DC.B	%01011110
        DC.B	%01100110
        DC.B	%00100110
        DC.B	%00011010

	; Esmeralda Sprites
Damsel0
	DC.B	%00000000
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111100
        DC.B	%00111100
        DC.B	%00111010
        DC.B	%00110010
        DC.B	%01111100
        DC.B	%00111000
        DC.B	%00010100
        DC.B	%00110100
        DC.B	%00111000
Damsel1
        DC.B	%00000000
        DC.B	%00111110
        DC.B	%00111100
        DC.B	%00111100
        DC.B	%00111100
        DC.B	%00111000
        DC.B	%00110010
        DC.B	%00110010
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00010100
        DC.B	%00110100
        DC.B	%00111000
Damsel2
	DC.B	%00000000
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111100
        DC.B	%00111100
        DC.B	%00111000
        DC.B	%00111010
        DC.B	%00110010
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00010010
        DC.B	%00110100
        DC.B	%00111000
Damsel3
	DC.B	%00000000
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111110
        DC.B	%00111100
        DC.B	%00111100
        DC.B	%00111010
        DC.B	%00110010
        DC.B	%01111100
        DC.B	%00111010
        DC.B	%00010010
        DC.B	%00110100
        DC.B	%00111000	

	; Lever & Bell Sprites
Lever1
	DC.B	%00000010	
	DC.B	%00000010	
	DC.B	%00000010
        DC.B    %11110010
        DC.B    %11110010	
Lever2       
        DC.B    %11010010
	DC.B	%00001110                       
        DC.B    %00111110
        DC.B    %00010010
        DC.B    %00010010	
Bell1
        DC.B    %11010010
        DC.B    %00001110	
        DC.B    %00111110
	DC.B	%11100010
        DC.B    %00101110
Bell2
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110	
	DC.B	%11010010
        DC.B    %00101110
Bell3
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110
	DC.B	%11000010
        DC.B    %00101110
Bell4
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110	
	DC.B	%10110010
        DC.B    %00101110
        DC.B    %00001010
        DC.B    %00011010
        DC.B    %00000110

	; Missile Sprites
Missile1				
	DC.B	%00000000			
	DC.B	%00000000	
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%00010110
	DC.B	%11110110
Missile2
	DC.B	%00000000
	DC.B	%00000000			
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%11110110
	DC.B	%00010110
Missile3
	DC.B	%00000000
	DC.B	%00000000		
	DC.B	%00000000
	DC.B	%00000100	
	DC.B	%00010110
	DC.B	%00010110
	DC.B	%00000000
	DC.B	%00000000
								
	ORG	$FF00	
		
        ; HMOVE Fine Tune Table	
FineTune
        DC.B    %01100010       ; + 6 (Left)
        DC.B    %01010010       ; + 5
        DC.B    %01000010
        DC.B    %00110010
        DC.B    %00100010
        DC.B    %00010010
        DC.B    %00000010       ; Centre
        DC.B    %11110010
        DC.B    %11100010
        DC.B    %11010010
        DC.B    %11000010
        DC.B    %10110010
        DC.B    %10100010
        DC.B    %10010010       ; - 7
        DC.B    %10000010       ; - 8 (Right)
FineTuneEnd = FineTune - 241	
	
	; Font Data (For LEVEL:	and GAMEOVER Text)
Font
Space
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000		
LE
        DC.B	%11101110
        DC.B	%10001000
        DC.B	%10001100
        DC.B	%10001000
        DC.B	%10001110
VE
        DC.B	%01001110
        DC.B	%01001000
        DC.B	%10101100
        DC.B	%10101000
        DC.B	%10101110
L
        DC.B	%11100000
        DC.B	%10000110
        DC.B	%10000000
        DC.B	%10000110
        DC.B	%10000000
GA
        DC.B	%01101010
        DC.B	%10101010
        DC.B	%10101110
        DC.B	%10001010
        DC.B	%01100100
ME
        DC.B	%10101110
        DC.B	%10101000
        DC.B	%10101100
        DC.B	%11101000
        DC.B	%10101110
OV
        DC.B	%01000100
        DC.B	%10100100
        DC.B	%10101010
        DC.B	%10101010
        DC.B	%01001010
ER
        DC.B	%11101010
        DC.B	%10001010
        DC.B	%11001100
        DC.B	%10001010
        DC.B	%11101100

	; Overlapping Numbers (By Thomas Jentzsch)
Digit
	DC.B	<Zero, <One, <Two, <Three, <Four
	DC.B	<Five, <Six, <Seven, <Eight, <Nine	
One
	DC.B	%00001100
Seven
	DC.B	%00001100
	DC.B	%00001100
Four
	DC.B	%00001100
	DC.B	%00001100
Zero
	DC.B	%01111000
	DC.B	%11001100
	DC.B	%11001100
	DC.B	%11001100
Three
	DC.B	%01111000
	DC.B	%00001100
Nine
	DC.B	%01111000
	DC.B	%00001100
Eight
	DC.B	%01111000
	DC.B	%11001100
Six
	DC.B	%01111000
	DC.B	%11001100
Two
	DC.B	%01111000
	DC.B	%11000000
Five
	DC.B	%01111000
	DC.B	%00001100
	DC.B	%01111000
	DC.B	%11000000
	DC.B	%01111000
	
	; Bell Sprites (Padded to 8 bytes)
	; NOTE:	HMove Timings are Delayed By 1 Line		
        ORG	($FF00 + KERNH)
	; Sprite Data 2
StartSprites2			
	; Knight Left/Right Sprites
Knight0
	DC.B	%00000000
        DC.B	%10101100
        DC.B	%10001100
        DC.B	%10111100
        DC.B	%10101110
        DC.B	%10101110
        DC.B	%10000110
        DC.B	%10101110
        DC.B	%10111110
        DC.B	%11101100
        DC.B	%10000100
        DC.B	%00111100
        DC.B	%00011000
Knight1
	DC.B	%00000000	
        DC.B	%00011000
        DC.B	%10011000
        DC.B	%10111100
        DC.B	%10101100
        DC.B	%10101101
        DC.B	%10000101
        DC.B	%10101111
        DC.B	%10111110
        DC.B	%10101100
        DC.B	%11000100
        DC.B	%10111100
        DC.B	%00011000
Knight2
	DC.B	%00000000	
        DC.B	%00110100
        DC.B	%00110000
        DC.B	%10111100
        DC.B	%10101100
        DC.B	%10101101
        DC.B	%10000101
        DC.B	%10101111
        DC.B	%10111110
        DC.B	%10101100
        DC.B	%10000100
        DC.B	%11111100
        DC.B	%10011000
Knight3
	DC.B	%00000000	
        DC.B	%00011000
        DC.B	%10011000
        DC.B	%10111100
        DC.B	%10101110
        DC.B	%10101110
        DC.B	%10000110
        DC.B	%10101110
        DC.B	%10111110
        DC.B	%10101100
        DC.B	%11000100
        DC.B	%10111100
        DC.B	%00011000
	; Knight Up/Down Sprites
Knight4
 	DC.B	%00000000	
        DC.B	%00001100
        DC.B	%01101100
        DC.B	%11111100
        DC.B	%11101100
        DC.B	%10101101
        DC.B	%10000101
        DC.B	%10101111
        DC.B	%10111110
        DC.B	%10101100
        DC.B	%10000100
        DC.B	%11111100
        DC.B	%10011000
Knight5
	DC.B	%00000000	
        DC.B	%10110000
        DC.B	%10110110
        DC.B	%10111110
        DC.B	%10110110
        DC.B	%10110100
        DC.B	%10100000
        DC.B	%10110110
        DC.B	%10111111
        DC.B	%11110101
        DC.B	%10100001
        DC.B	%00111100
        DC.B	%00011000
	; Knight Thrusting Sprites
Knight6
	DC.B	%00000000	
        DC.B	%01010110
        DC.B	%01000110
        DC.B	%01011110
        DC.B	%01010111
        DC.B	%01010111
        DC.B	%01000011
        DC.B	%01010111
        DC.B	%01011111
        DC.B	%01110110
        DC.B	%01000010
        DC.B	%00011110
        DC.B	%00001100
        DC.B	%00000000
        DC.B	%00000000
        DC.B	%00000000
        DC.B	%00000000
Knight7
	DC.B	%00000000	
	DC.B	%00010110
        DC.B	%00000110
        DC.B	%00011110
        DC.B	%10010111
        DC.B	%10010111
        DC.B	%10000011
        DC.B	%10010111
        DC.B	%01011111
        DC.B	%01010110
        DC.B	%01000010
        DC.B	%01011110
        DC.B	%01101100
	DC.B	%01000000
	DC.B	%00000000
	DC.B	%00000000
	DC.B	%00000000
Knight8
	DC.B	%00000000
	DC.B	%00010110
	DC.B	%00000110
	DC.B	%00011110
	DC.B	%00010111
	DC.B	%00010111
	DC.B	%00000011
	DC.B	%10010111
	DC.B	%10011111
	DC.B	%10010110
	DC.B	%01000010
	DC.B	%01011110
	DC.B	%01001100
	DC.B	%00100000
	DC.B	%00100000
	DC.B	%00011000
	DC.B	%00010000

	; Player Frames
HorizFrames
	DC.B	<Player0,<Player1,<Player0,<Player2
VertFrames
	DC.B	<Player3,<Player4	

	; Knight Frames
KnightHorizFrames
	DC.B	<Knight0,<Knight1,<Knight2,<Knight3
KnightVertFrames
	DC.B	<Knight4,<Knight5
KnightSpearFrames
	DC.B	<Knight6,<Knight7,<Knight8,<Knight7
	
	; Damsel Frames
DamselFrames
	DC.B	<Damsel0,<Damsel1,<Damsel2,<Damsel3	
		
	; Leave Blank For Supercharger Compatibility
	ORG	$FFF8,0
		
	; ROM Start Pointers
        ORG	$FFFA,0
	.word Start
        .word Start
        .word Start
